#!/bin/bash
set -e

echo "Patching log4j2 configuration on components to mitigate CVE-2021-44228..."

#core services
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/bd-management/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/bd-management/config.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=50.0"#-XX:MaxRAMPercentage=50.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/bd-management/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=50.0"#-XX:MaxRAMPercentage=50.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/bd-management/config.yaml 2>/dev/null || true

sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/data-server/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/data-server/config.yaml 2>/dev/null || true

sed -i '11s#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/data-catalog-server/config-vm.yaml 2>/dev/null || true
sed -i '11s#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/data-catalog-server/config.yaml 2>/dev/null || true

sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/faz-web/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/faz-web/config.yaml 2>/dev/null || true

sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/ingestion-server/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/ingestion-server/config.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/ingestion-server-ng/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:MaxRAMPercentage=80.0"#-XX:MaxRAMPercentage=80.0 -Dlog4j2.formatMsgNoLookups=true"#g' /fazbd/k8s/services/ingestion-server-ng/config.yaml 2>/dev/null || true

sed -i 's#-XX:+UseG1GC -XX:G1HeapRegionSize=16M#-XX:+UseG1GC -Dlog4j2.formatMsgNoLookups=true -XX:G1HeapRegionSize=16M#g' /fazbd/k8s/services/streaming-etl/config.yaml 2>/dev/null || true
sed -i 's#-XX:+UseG1GC -XX:G1HeapRegionSize=16M#-XX:+UseG1GC -Dlog4j2.formatMsgNoLookups=true -XX:G1HeapRegionSize=16M#g' /fazbd/k8s/services/streaming-etl/config-vm.yaml 2>/dev/null || true
sed -i 's#-XX:+UseG1GC -XX:G1HeapRegionSize=16M#-XX:+UseG1GC -Dlog4j2.formatMsgNoLookups=true -XX:G1HeapRegionSize=16M#g' /fazbd/k8s/services/streaming-etl-ng/config.yaml 2>/dev/null || true
sed -i 's#-XX:+UseG1GC -XX:G1HeapRegionSize=16M#-XX:+UseG1GC -Dlog4j2.formatMsgNoLookups=true -XX:G1HeapRegionSize=16M#g' /fazbd/k8s/services/streaming-etl-ng/config-vm.yaml 2>/dev/null || true

echo
echo
echo "Rolling out configuration changes to core services... (this will take about 5mins)"
fazbdadm run fazbd_init_service silent > /dev/null 2>&1

helm uninstall streaming-etl > /dev/null 2>&1

#data frameworks (kafka, yarn (only for <=646), hive metastore, hdfs, zookeeper)
bd_hosts=$(consul members | grep "\." | awk '{print $2}'|awk -F: '{print $1}')
for host in $bd_hosts; do
  ssh-keygen -R "$host" > /dev/null 2>&1
  ssh -o StrictHostKeyChecking=no -T -t "$host" bash <<EOF
        sed -i 's#export KAFKA_LOG4J_OPTS="-Dlog4j.configuration=file:\\\$base_dir/../config/log4j.properties"#export KAFKA_LOG4J_OPTS="-Dlog4j2.formatMsgNoLookups=true -Dlog4j.configuration=file:\\\$base_dir/../config/log4j.properties"#g' /mnt/config/consul_template/kafka-server-start.sh.ctmpl 2>/dev/null || true
        sed -i 's#-XX:HeapDumpPath=/tmp/yarn_yarn-NODEMANAGER.hprof"#-XX:HeapDumpPath=/tmp/yarn_yarn-NODEMANAGER.hprof -Dlog4j2.formatMsgNoLookups=true"#g' /etc/hadoop/conf/yarn-env.sh 2>/dev/null || true
        sed -i 's/#export YARN_RESOURCEMANAGER_OPTS=/export YARN_RESOURCEMANAGER_OPTS="-Dlog4j2.formatMsgNoLookups=true"/g' /etc/hadoop/conf/yarn-env.sh 2>/dev/null || true
        sed -i "s/# export HIVE_AUX_JARS_PATH=/export HADOOP_OPTS=\"\\\$HADOOP_OPTS -Dlog4j2.formatMsgNoLookups=true\"/g" /etc/hive/conf/hive-env.sh 2>/dev/null || true
        sed -i "s|# NOTE: this files overwrites varibales from /etc/default/hadoop\*|export HADOOP_OPTS=\"\\\$HADOOP_OPTS -Dlog4j2.formatMsgNoLookups=true\"|g" /etc/hadoop/conf/hadoop-env.sh 2>/dev/null || true
        sed -i "s|# export SERVER_JVMFLAGS=\"\"|export SERVER_JVMFLAGS=\"\\\$SERVER_JVMFLAGS -Dlog4j2.formatMsgNoLookups=true\"|g" /etc/default/zookeeper 2>/dev/null || true
        sed -i "s|export SPARK_HISTORY_OPTS=\"\\\$SPARK_HISTORY_OPTS|export SPARK_HISTORY_OPTS=\"-Dlog4j2.formatMsgNoLookups=true \\\$SPARK_HISTORY_OPTS|g" /etc/default/spark 2>/dev/null || true
EOF
done

echo
echo 
echo "Rolling out configuration changes to data frameworks... (this will take about 5mins)"

cat > /fazbd/ansible-fazbd-hadoop/restart_services.yaml <<EOF
---
- name: Restart zookeeper
  hosts: zookeeper
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=zookeeper-server state=restarted enabled=yes

- name: Restart Kafka Master
  hosts: kafka_broker
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=kafka-server state=restarted enabled=yes

- name: Restart Hive Metastore
  hosts: hive_metastore
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hive-metastore state=restarted enabled=yes

- name: Restart HDFS namenode
  hosts: hdfs_namenode
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hadoop-hdfs-namenode state=restarted enabled=yes

- name: Restart zkfc
  hosts: hdfs_namenode
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hadoop-hdfs-zkfc state=restarted enabled=yes
      when:  groups['hdfs_namenode']|count > 1

- name: Restart HDFS journalnodes
  hosts: journalnodes
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hadoop-hdfs-journalnode state=restarted enabled=yes

- name: Restart HDFS datanode
  hosts: hdfs_datanode
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hadoop-hdfs-datanode state=restarted enabled=yes

- name: Restart Yarn ResourceManager (<=6.4.6)
  hosts: hdfs_namenode
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hadoop-yarn-resourcemanager state=restarted enabled=yes
      when: "'yarn_resource_manager' in groups"

- name: Restart Yarn Nodemanager (<=6.4.6)
  hosts: hdfs_datanode
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hadoop-yarn-nodemanager state=restarted enabled=yes
      when: "'yarn_resource_manager' in groups"

- name: Restart Hive Server2 (<=6.4.6)
  hosts: hdfs_namenode
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=hive-server2 state=restarted enabled=yes
      when: "'yarn_resource_manager' in groups"

- name: Restart Spark History Serve (<=6.4.6)
  hosts: spark_history_server
  gather_facts: no
  tasks:
    - name: restart services
      tags: service
      service: name=spark-history-server state=restarted enabled=yes
      when: "'spark_history_server' in groups"
EOF

fazbdadm run playbook restart_services.yaml

rm /fazbd/ansible-fazbd-hadoop/restart_services.yaml

fazbdadm run fazbd_init_service silent > /dev/null 2>&1

echo
echo
echo "Finished configuring log4j2 for all components."